@@ -19,7 +19,9 @@ enum class DateTimeOutputFormat {
     DateOnly, /**< date only */
     TimeOnly, /**< time only */
     DateTimeAndWeekday, /**< date with weekday and time */
-    DateTimeAndShortWeekday /**< date with abbreviated weekday and time */
+    DateTimeAndShortWeekday, /**< date with abbreviated weekday and time */
+    Iso, /**< ISO format like DateTime::toIsoString() */
+    IsoOmittingDefaultComponents, /**< ISO format like DateTime::toIsoString() omitting default components, e.g. just "2017" instead of "2017-01-01T00:00:00" */
 };
 
 /*!
@@ -60,8 +62,10 @@ public:
     static std::pair<DateTime, TimeSpan> fromIsoString(const char *str);
     static DateTime fromIsoStringGmt(const char *str);
     static DateTime fromIsoStringLocal(const char *str);
-    static DateTime fromTimeStamp(time_t timeStamp);
-    constexpr static DateTime fromTimeStampGmt(time_t timeStamp);
+    static DateTime fromTimeStamp(std::time_t timeStamp);
+    constexpr static DateTime fromTimeStampGmt(std::time_t timeStamp);
+    template <typename TimePoint> static DateTime fromChronoTimePoint(TimePoint timePoint);
+    template <typename TimePoint> constexpr static DateTime fromChronoTimePointGmt(TimePoint timePoint);
 
     constexpr std::uint64_t &ticks();
     constexpr std::uint64_t totalTicks() const;
@@ -83,7 +87,10 @@ public:
     constexpr bool isSameDay(const DateTime &other) const;
     std::string toString(DateTimeOutputFormat format = DateTimeOutputFormat::DateAndTime, bool noMilliseconds = false) const;
     void toString(std::string &result, DateTimeOutputFormat format = DateTimeOutputFormat::DateAndTime, bool noMilliseconds = false) const;
+    std::string toIsoStringWithCustomDelimiters(
+        TimeSpan timeZoneDelta = TimeSpan(), char dateDelimiter = '-', char timeDelimiter = ':', char timeZoneDelimiter = ':') const;
     std::string toIsoString(TimeSpan timeZoneDelta = TimeSpan()) const;
+    constexpr std::time_t toTimeStamp() const;
     static const char *printDayOfWeek(DayOfWeek dayOfWeek, bool abbreviation = false);
 
     static constexpr DateTime eternity();
@@ -169,10 +176,7 @@ inline DateTime DateTime::fromTime(int hour, int minute, int second, double mill
  */
 inline DateTime DateTime::fromDateAndTime(int year, int month, int day, int hour, int minute, int second, double millisecond)
 {
-    if (std::uint64_t ticks = dateToTicks(year, month, day)) {
-        return DateTime(ticks + timeToTicks(hour, minute, second, millisecond));
-    }
-    return DateTime();
+    return DateTime(dateToTicks(year, month, day) + timeToTicks(hour, minute, second, millisecond));
 }
 
 /*!
@@ -220,6 +224,26 @@ constexpr inline DateTime DateTime::fromTimeStampGmt(std::time_t timeStamp)
     return DateTime(DateTime::unixEpochStart().totalTicks() + static_cast<std::uint64_t>(timeStamp) * TimeSpan::ticksPerSecond);
 }
 
+/*!
+ * \brief Constructs a new DateTime object with the local time from the specified std::chrono::time_point.
+ * \remarks Works only with time points of std::chrono::system_clock so far. C++20 will fix this. Until then this function
+ *          should be considered experimental.
+ */
+template <typename TimePoint> inline DateTime DateTime::fromChronoTimePoint(TimePoint timePoint)
+{
+    return DateTime::fromTimeStamp(decltype(timePoint)::clock::to_time_t(timePoint));
+}
+
+/*!
+ * \brief Constructs a new DateTime object with the GMT time from the specified std::chrono::time_point.
+ * \remarks Works only with time points of std::chrono::system_clock so far. C++20 will fix this. Until then this function
+ *          should be considered experimental.
+ */
+template <typename TimePoint> constexpr DateTime DateTime::fromChronoTimePointGmt(TimePoint timePoint)
+{
+    return DateTime::fromTimeStampGmt(decltype(timePoint)::clock::to_time_t(timePoint));
+}
+
 /*!
  * \brief Returns a mutable reference to the total ticks.
  */
@@ -396,6 +420,14 @@ inline std::string DateTime::toString(DateTimeOutputFormat format, bool noMillis
     return result;
 }
 
+/*!
+ * \brief Returns the UNIX timestamp for the current instance.
+ */
+constexpr std::time_t DateTime::toTimeStamp() const
+{
+    return (totalTicks() - DateTime::unixEpochStart().totalTicks()) / TimeSpan::ticksPerSecond;
+}
+
 /*!
  * \brief Constructs a new instance of the DateTime class with the maximal number of ticks.
  */
