@@ -14,6 +14,7 @@
 #include <memory>
 #include <sstream>
 #include <string>
+#include <system_error>
 #include <vector>
 
 namespace CppUtilities {
@@ -49,6 +50,8 @@ CPP_UTILITIES_EXPORT StringData convertUtf8ToLatin1(const char *inputBuffer, std
 
 #ifdef PLATFORM_WINDOWS
 using WideStringData = std::pair<std::unique_ptr<wchar_t[]>, int>;
+CPP_UTILITIES_EXPORT WideStringData convertMultiByteToWide(std::error_code &ec, const char *inputBuffer, int inputBufferSize = -1);
+CPP_UTILITIES_EXPORT WideStringData convertMultiByteToWide(std::error_code &ec, const std::string &inputBuffer);
 CPP_UTILITIES_EXPORT WideStringData convertMultiByteToWide(const char *inputBuffer, int inputBufferSize = -1);
 CPP_UTILITIES_EXPORT WideStringData convertMultiByteToWide(const std::string &inputBuffer);
 #endif
@@ -212,9 +215,11 @@ template <typename StringType> bool startsWith(const StringType &str, const Stri
     if (str.size() < phrase.size()) {
         return false;
     }
-    for (auto stri = str.cbegin(), strend = str.cend(), phrasei = phrase.cbegin(), phraseend = phrase.cend(); stri != strend; ++stri, ++phrasei) {
+    for (auto stri = str.cbegin(), strend = str.cend(), phrasei = phrase.cbegin(), phraseend = phrase.cend();; ++stri, ++phrasei) {
         if (phrasei == phraseend) {
             return true;
+        } else if (stri == strend) {
+            return false;
         } else if (*stri != *phrasei) {
             return false;
         }
@@ -227,9 +232,11 @@ template <typename StringType> bool startsWith(const StringType &str, const Stri
  */
 template <typename StringType> bool startsWith(const StringType &str, const typename StringType::value_type *phrase)
 {
-    for (auto stri = str.cbegin(), strend = str.cend(); stri != strend; ++stri, ++phrase) {
+    for (auto stri = str.cbegin(), strend = str.cend();; ++stri, ++phrase) {
         if (!*phrase) {
             return true;
+        } else if (stri == strend) {
+            return false;
         } else if (*stri != *phrase) {
             return false;
         }
@@ -237,6 +244,39 @@ template <typename StringType> bool startsWith(const StringType &str, const type
     return false;
 }
 
+/*!
+ * \brief Returns whether \a str ends with \a phrase.
+ */
+template <typename StringType> bool endsWith(const StringType &str, const StringType &phrase)
+{
+    if (str.size() < phrase.size()) {
+        return false;
+    }
+    for (auto stri = str.cend() - phrase.size(), strend = str.cend(), phrasei = phrase.cbegin(); stri != strend; ++stri, ++phrasei) {
+        if (*stri != *phrasei) {
+            return false;
+        }
+    }
+    return true;
+}
+
+/*!
+ * \brief Returns whether \a str ends with \a phrase.
+ */
+template <typename StringType> bool endsWith(const StringType &str, const typename StringType::value_type *phrase)
+{
+    const auto phraseSize = std::strlen(phrase);
+    if (str.size() < phraseSize) {
+        return false;
+    }
+    for (auto stri = str.cend() - phraseSize, strend = str.cend(); stri != strend; ++stri, ++phrase) {
+        if (*stri != *phrase) {
+            return false;
+        }
+    }
+    return true;
+}
+
 /*!
  * \brief Returns whether \a str contains the specified \a substrings.
  * \remarks The \a substrings must occur in the specified order.
