@@ -50,8 +50,8 @@ class ArgumentParserTests : public TestFixture {
     CPPUNIT_TEST_SUITE_END();
 
 public:
-    void setUp();
-    void tearDown();
+    void setUp() override;
+    void tearDown() override;
 
     void testArgument();
     void testParsing();
