@@ -13,6 +13,10 @@ using namespace CppUtilities;
 #include <random>
 #include <sstream>
 
+#ifdef CPP_UTILITIES_USE_STANDARD_FILESYSTEM
+#include <filesystem>
+#endif
+
 using namespace std;
 
 using namespace CPPUNIT_NS;
@@ -311,10 +315,24 @@ void ConversionTests::testStringConversions()
     CPPUNIT_ASSERT_EQUAL("findOrReplace()"s, findReplaceTest);
 
     // startsWith()
-    CPPUNIT_ASSERT(!startsWith<string>(findReplaceTest, "findAnd"));
-    CPPUNIT_ASSERT(startsWith<string>(findReplaceTest, "findOr"));
-    CPPUNIT_ASSERT(!startsWith<string>(findReplaceTest, "findAnd"s));
-    CPPUNIT_ASSERT(startsWith<string>(findReplaceTest, "findOr"s));
+    CPPUNIT_ASSERT(!startsWith(findReplaceTest, "findAnd"));
+    CPPUNIT_ASSERT(startsWith(findReplaceTest, "findOr"));
+    CPPUNIT_ASSERT(!startsWith(findReplaceTest, "findAnd"s));
+    CPPUNIT_ASSERT(startsWith(findReplaceTest, "findOr"s));
+    CPPUNIT_ASSERT(startsWith("test"s, "test"s));
+    CPPUNIT_ASSERT(startsWith("test"s, "test"));
+    CPPUNIT_ASSERT(!startsWith("test"s, "tests"s));
+    CPPUNIT_ASSERT(!startsWith("test"s, "tests"));
+
+    // endsWith()
+    CPPUNIT_ASSERT(!endsWith(findReplaceTest, "AndReplace()"));
+    CPPUNIT_ASSERT(endsWith(findReplaceTest, "OrReplace()"));
+    CPPUNIT_ASSERT(!endsWith(findReplaceTest, "AndReplace()"s));
+    CPPUNIT_ASSERT(endsWith(findReplaceTest, "OrReplace()"s));
+    CPPUNIT_ASSERT(endsWith("test"s, "test"s));
+    CPPUNIT_ASSERT(endsWith("test"s, "test"));
+    CPPUNIT_ASSERT(!endsWith("test"s, " test"s));
+    CPPUNIT_ASSERT(!endsWith("test"s, " test"));
 
     // containsSubstrings()
     CPPUNIT_ASSERT(containsSubstrings<string>("this string contains foo and bar", { "foo", "bar" }));
@@ -365,25 +383,77 @@ void ConversionTests::testStringConversions()
     CPPUNIT_ASSERT_EQUAL("16 GiB/s"s, bitrateToString(128.0 * 1e6, true));
 }
 
-string functionTakingString(const string &str)
-{
-    return str;
-}
+/// \cond
+
+struct ConvertibleToString {
+    operator std::string() const;
+};
+
+struct StringThatDoesNotLikeToBeCopiedOrMoved : public std::string {
+    explicit StringThatDoesNotLikeToBeCopiedOrMoved(const char *value)
+        : std::string(value)
+    {
+    }
+    [[noreturn]] StringThatDoesNotLikeToBeCopiedOrMoved(const StringThatDoesNotLikeToBeCopiedOrMoved &other)
+        : std::string(other)
+    {
+        CPPUNIT_FAIL("attempt to copy string: " + other);
+    }
+    [[noreturn]] StringThatDoesNotLikeToBeCopiedOrMoved(StringThatDoesNotLikeToBeCopiedOrMoved &&other)
+        : std::string(std::move(other))
+    {
+        CPPUNIT_FAIL("attempt to move string: " + other);
+    }
+};
+
+/// \endcond
 
 void ConversionTests::testStringBuilder()
 {
+    // check whether type traits work as expected
+    static_assert(Helper::IsStringType<std::string, std::string>::value);
+    static_assert(!Helper::IsStringType<std::string, std::wstring>::value);
+    static_assert(Helper::IsStringType<std::wstring, std::wstring>::value);
+    static_assert(Helper::IsStringViewType<std::string, std::string_view>::value);
+    static_assert(!Helper::IsStringViewType<std::wstring, std::string_view>::value);
+    static_assert(Helper::IsStringViewType<std::wstring, std::wstring_view>::value);
+    static_assert(Helper::IsConvertibleToConstStringRef<std::string, ConvertibleToString>::value);
+#ifdef CPP_UTILITIES_USE_STANDARD_FILESYSTEM
+    static_assert(!Helper::IsConvertibleToConstStringRef<std::filesystem::path::string_type, std::filesystem::path>::value,
+        "conversion via native() preferred");
+#endif
+    static_assert(
+        !Helper::IsConvertibleToConstStringRef<std::string, std::string>::value, "yes, in this context this should not be considered convertible");
+    static_assert(!Helper::IsConvertibleToConstStringRef<std::wstring, ConvertibleToString>::value);
+#ifdef CPP_UTILITIES_USE_STANDARD_FILESYSTEM
+    static_assert(Helper::IsConvertibleToConstStringRefViaNative<std::filesystem::path::string_type, std::filesystem::path>::value);
+#endif
+    static_assert(!Helper::IsConvertibleToConstStringRefViaNative<std::string, std::string>::value);
+
     // conversion of string-tuple to string (the actual string builder)
     const tuple<const char *, string, int, const char *> tuple("string1", "string2", 1234, "string3");
-    CPPUNIT_ASSERT_EQUAL(string("string1string21234string3"), tupleToString(tuple));
-    CPPUNIT_ASSERT_EQUAL(string("foobarfoo2bar2"), tupleToString(string("foo") % "bar" % string("foo2") % "bar2"));
-    CPPUNIT_ASSERT_EQUAL(string("v2.3.0"), argsToString("v2.", 3, '.', 0));
+    CPPUNIT_ASSERT_EQUAL("string1string21234string3"s, tupleToString(tuple));
+    CPPUNIT_ASSERT_EQUAL("foobarfoo2bar2"s, tupleToString("foo"s % "bar" % "foo2"s % "bar2"));
+    CPPUNIT_ASSERT_EQUAL("v2.3.0"s, argsToString("v2.", 3, '.', 0));
+    CPPUNIT_ASSERT_EQUAL("v2.3.0"s, argsToString('v', make_tuple(2, '.', 3, '.', 0)));
+#ifdef CPP_UTILITIES_USE_STANDARD_FILESYSTEM
+    if constexpr (std::is_same_v<std::filesystem::path::value_type, std::string::value_type>) {
+        CPPUNIT_ASSERT_EQUAL("path: foo"s, argsToString("path: ", std::filesystem::path("foo")));
+    }
+#endif
 
     // construction of string-tuple and final conversion to string works
-    CPPUNIT_ASSERT_EQUAL_MESSAGE(
-        "result can be passed to any function taking a std::string"s, "123456789"s, functionTakingString("12" % string("34") % '5' % 67 + "89"));
+    CPPUNIT_ASSERT_EQUAL_MESSAGE("result can be passed to any function taking a std::string"s, "123456789"s, "12" % string("34") % '5' % 67 + "89");
     constexpr double velocityExample = 27.0;
     CPPUNIT_ASSERT_EQUAL_MESSAGE("real-word example"s, "velocity: 27 km/h (7.5 m/s)"s,
-        functionTakingString("velocity: " % numberToString(velocityExample) % " km/h (" % numberToString(velocityExample / 3.6) + " m/s)"));
+        "velocity: " % numberToString(velocityExample) % " km/h (" % numberToString(velocityExample / 3.6) + " m/s)");
     CPPUNIT_ASSERT_EQUAL_MESSAGE(
         "regular + operator still works (no problems with ambiguity)"s, "regular + still works"s, "regular"s + " + still works");
+    CPPUNIT_ASSERT_EQUAL_MESSAGE("using string_view", "foobar123"s, "foo"sv % "bar"sv + 123);
+
+    // check that for the internal tuple construction no copies are made
+    StringThatDoesNotLikeToBeCopiedOrMoved str(" happen ");
+    const StringThatDoesNotLikeToBeCopiedOrMoved str2("for this");
+    CPPUNIT_ASSERT_EQUAL("no copy/move should happen for this!"s,
+        argsToString(StringThatDoesNotLikeToBeCopiedOrMoved("no copy/move should"), str, str2, StringThatDoesNotLikeToBeCopiedOrMoved("!")));
 }
