@@ -45,8 +45,10 @@ class IoTests : public TestFixture {
     CPPUNIT_TEST(testBitReader);
     CPPUNIT_TEST(testPathUtilities);
     CPPUNIT_TEST(testIniFile);
+    CPPUNIT_TEST(testAdvancedIniFile);
     CPPUNIT_TEST(testCopy);
     CPPUNIT_TEST(testReadFile);
+    CPPUNIT_TEST(testWriteFile);
     CPPUNIT_TEST(testAnsiEscapeCodes);
 #ifdef CPP_UTILITIES_USE_NATIVE_FILE_BUFFER
     CPPUNIT_TEST(testNativeFileStream);
@@ -54,16 +56,18 @@ class IoTests : public TestFixture {
     CPPUNIT_TEST_SUITE_END();
 
 public:
-    void setUp();
-    void tearDown();
+    void setUp() override;
+    void tearDown() override;
 
     void testBinaryReader();
     void testBinaryWriter();
     void testBitReader();
     void testPathUtilities();
     void testIniFile();
+    void testAdvancedIniFile();
     void testCopy();
     void testReadFile();
+    void testWriteFile();
     void testAnsiEscapeCodes();
 #ifdef CPP_UTILITIES_USE_NATIVE_FILE_BUFFER
     void testNativeFileStream();
@@ -90,45 +94,46 @@ void IoTests::testBinaryReader()
     testFile.exceptions(ios_base::failbit | ios_base::badbit);
     testFile.open(testFilePath("some_data"), ios_base::in | ios_base::binary);
     BinaryReader reader(&testFile);
-    CPPUNIT_ASSERT_EQUAL(reader.readStreamsize(), static_cast<istream::pos_type>(398));
-    CPPUNIT_ASSERT(reader.readUInt16LE() == 0x0102u);
-    CPPUNIT_ASSERT(reader.readUInt16BE() == 0x0102u);
-    CPPUNIT_ASSERT(reader.readUInt24LE() == 0x010203u);
-    CPPUNIT_ASSERT(reader.readUInt24BE() == 0x010203u);
-    CPPUNIT_ASSERT(reader.readUInt32LE() == 0x01020304u);
-    CPPUNIT_ASSERT(reader.readUInt32BE() == 0x01020304u);
-    CPPUNIT_ASSERT(reader.readUInt40LE() == 0x0102030405u);
-    CPPUNIT_ASSERT(reader.readUInt40BE() == 0x0102030405u);
-    CPPUNIT_ASSERT(reader.readUInt56LE() == 0x01020304050607u);
-    CPPUNIT_ASSERT(reader.readUInt56BE() == 0x01020304050607u);
-    CPPUNIT_ASSERT(reader.readUInt64LE() == 0x0102030405060708u);
-    CPPUNIT_ASSERT(reader.readUInt64BE() == 0x0102030405060708u);
+    CPPUNIT_ASSERT_EQUAL(static_cast<istream::pos_type>(398), reader.readStreamsize());
+    CPPUNIT_ASSERT_EQUAL(static_cast<std::uint16_t>(0x0102u), reader.readUInt16LE());
+    CPPUNIT_ASSERT_EQUAL(static_cast<istream::pos_type>(396), reader.readRemainingBytes());
+    CPPUNIT_ASSERT_EQUAL(static_cast<std::uint16_t>(0x0102u), reader.readUInt16BE());
+    CPPUNIT_ASSERT_EQUAL(0x010203u, reader.readUInt24LE());
+    CPPUNIT_ASSERT_EQUAL(0x010203u, reader.readUInt24BE());
+    CPPUNIT_ASSERT_EQUAL(0x01020304u, reader.readUInt32LE());
+    CPPUNIT_ASSERT_EQUAL(0x01020304u, reader.readUInt32BE());
+    CPPUNIT_ASSERT_EQUAL(0x0102030405u, reader.readUInt40LE());
+    CPPUNIT_ASSERT_EQUAL(0x0102030405u, reader.readUInt40BE());
+    CPPUNIT_ASSERT_EQUAL(0x01020304050607u, reader.readUInt56LE());
+    CPPUNIT_ASSERT_EQUAL(0x01020304050607u, reader.readUInt56BE());
+    CPPUNIT_ASSERT_EQUAL(0x0102030405060708u, reader.readUInt64LE());
+    CPPUNIT_ASSERT_EQUAL(0x0102030405060708u, reader.readUInt64BE());
     testFile.seekg(0);
-    CPPUNIT_ASSERT(reader.readInt16LE() == 0x0102);
-    CPPUNIT_ASSERT(reader.readInt16BE() == 0x0102);
-    CPPUNIT_ASSERT(reader.readInt24LE() == 0x010203);
-    CPPUNIT_ASSERT(reader.readInt24BE() == 0x010203);
-    CPPUNIT_ASSERT(reader.readInt32LE() == 0x01020304);
-    CPPUNIT_ASSERT(reader.readInt32BE() == 0x01020304);
-    CPPUNIT_ASSERT(reader.readInt40LE() == 0x0102030405);
-    CPPUNIT_ASSERT(reader.readInt40BE() == 0x0102030405);
-    CPPUNIT_ASSERT(reader.readInt56LE() == 0x01020304050607);
-    CPPUNIT_ASSERT(reader.readInt56BE() == 0x01020304050607);
-    CPPUNIT_ASSERT(reader.readInt64LE() == 0x0102030405060708);
-    CPPUNIT_ASSERT(reader.readInt64BE() == 0x0102030405060708);
-    CPPUNIT_ASSERT(reader.readFloat32LE() == 1.125);
-    CPPUNIT_ASSERT(reader.readFloat64LE() == 1.625);
-    CPPUNIT_ASSERT(reader.readFloat32BE() == 1.125);
-    CPPUNIT_ASSERT(reader.readFloat64BE() == 1.625);
-    CPPUNIT_ASSERT(reader.readBool() == false);
-    CPPUNIT_ASSERT(reader.readBool() == true);
-    CPPUNIT_ASSERT(reader.readString(3) == "abc");
-    CPPUNIT_ASSERT(reader.readLengthPrefixedString() == "ABC");
-    CPPUNIT_ASSERT(reader.readLengthPrefixedString()
-        == "01234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901"
-           "23456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123"
-           "45678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345"
-           "678901234567890123456789");
+    CPPUNIT_ASSERT_EQUAL(reader.readInt16LE(), static_cast<std::int16_t>(0x0102));
+    CPPUNIT_ASSERT_EQUAL(reader.readInt16BE(), static_cast<std::int16_t>(0x0102));
+    CPPUNIT_ASSERT_EQUAL(0x010203, reader.readInt24LE());
+    CPPUNIT_ASSERT_EQUAL(0x010203, reader.readInt24BE());
+    CPPUNIT_ASSERT_EQUAL(0x01020304, reader.readInt32LE());
+    CPPUNIT_ASSERT_EQUAL(0x01020304, reader.readInt32BE());
+    CPPUNIT_ASSERT_EQUAL(0x0102030405, reader.readInt40LE());
+    CPPUNIT_ASSERT_EQUAL(0x0102030405, reader.readInt40BE());
+    CPPUNIT_ASSERT_EQUAL(0x01020304050607, reader.readInt56LE());
+    CPPUNIT_ASSERT_EQUAL(0x01020304050607, reader.readInt56BE());
+    CPPUNIT_ASSERT_EQUAL(0x0102030405060708, reader.readInt64LE());
+    CPPUNIT_ASSERT_EQUAL(0x0102030405060708, reader.readInt64BE());
+    CPPUNIT_ASSERT_EQUAL(1.125f, reader.readFloat32LE());
+    CPPUNIT_ASSERT_EQUAL(1.625, reader.readFloat64LE());
+    CPPUNIT_ASSERT_EQUAL(1.125f, reader.readFloat32BE());
+    CPPUNIT_ASSERT_EQUAL(reader.readFloat64BE(), 1.625);
+    CPPUNIT_ASSERT_EQUAL(false, reader.readBool());
+    CPPUNIT_ASSERT_EQUAL(true, reader.readBool());
+    CPPUNIT_ASSERT_EQUAL("abc"s, reader.readString(3));
+    CPPUNIT_ASSERT_EQUAL(reader.readLengthPrefixedString(), "ABC"s);
+    CPPUNIT_ASSERT_EQUAL("01234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901"
+                         "23456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123"
+                         "45678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345"
+                         "678901234567890123456789"s,
+        reader.readLengthPrefixedString());
     CPPUNIT_ASSERT_EQUAL("def"s, reader.readTerminatedString());
     testFile.seekg(-4, ios_base::cur);
     CPPUNIT_ASSERT_EQUAL("def"s, reader.readTerminatedString(5, 0));
@@ -262,6 +267,7 @@ void IoTests::testPathUtilities()
     CPPUNIT_ASSERT_EQUAL("libc++utilities.so"s, fileName("C:\\libs\\libc++utilities.so"));
     CPPUNIT_ASSERT_EQUAL("libc++utilities.so"s, fileName("C:\\libs/libc++utilities.so"));
     CPPUNIT_ASSERT_EQUAL("libc++utilities.so"s, fileName("/usr/lib/libc++utilities.so"));
+    CPPUNIT_ASSERT_EQUAL("libc++utilities.so"s, fileName("libc++utilities.so"));
     CPPUNIT_ASSERT_EQUAL("/usr/lib/"s, directory("/usr/lib/libc++utilities.so"));
     CPPUNIT_ASSERT_EQUAL(string(), directory("libc++utilities.so"));
     CPPUNIT_ASSERT_EQUAL("C:\\libs\\"s, directory("C:\\libs\\libc++utilities.so"));
@@ -312,6 +318,74 @@ void IoTests::testIniFile()
     CPPUNIT_ASSERT(ini.data() == ini2.data());
 }
 
+/*!
+ * \brief Tests AdvancedIniFile.
+ */
+void IoTests::testAdvancedIniFile()
+{
+    // prepare reading test file
+    fstream inputFile;
+    inputFile.exceptions(ios_base::failbit | ios_base::badbit);
+    inputFile.open(testFilePath("pacman.conf"), ios_base::in);
+
+    // parse the test file
+    AdvancedIniFile ini;
+    ini.parse(inputFile);
+
+    // check whether scope data is as expected
+    CPPUNIT_ASSERT_EQUAL_MESSAGE("5 scopes (taking implicit empty section at the end into account)", 5_st, ini.sections.size());
+    auto options = ini.findSection("options");
+    CPPUNIT_ASSERT(options != ini.sectionEnd());
+    TESTUTILS_ASSERT_LIKE_FLAGS(
+        "comment block before section", "# Based on.*\n.*# GENERAL OPTIONS\n#\n"s, std::regex::extended, options->precedingCommentBlock);
+    CPPUNIT_ASSERT_EQUAL(7_st, options->fields.size());
+    CPPUNIT_ASSERT_EQUAL("HoldPkg"s, options->fields[0].key);
+    CPPUNIT_ASSERT_EQUAL("pacman glibc"s, options->fields[0].value);
+    CPPUNIT_ASSERT_MESSAGE("value present", options->fields[0].flags & IniFileFieldFlags::HasValue);
+    TESTUTILS_ASSERT_LIKE_FLAGS("comment block between section header and first field",
+        "# The following paths are.*\n.*#HookDir     = /etc/pacman\\.d/hooks/\n"s, std::regex::extended, options->fields[0].precedingCommentBlock);
+    CPPUNIT_ASSERT_EQUAL(""s, options->fields[0].followingInlineComment);
+    CPPUNIT_ASSERT_EQUAL("Foo"s, options->fields[1].key);
+    CPPUNIT_ASSERT_EQUAL("bar"s, options->fields[1].value);
+    CPPUNIT_ASSERT_MESSAGE("value present", options->fields[1].flags & IniFileFieldFlags::HasValue);
+    TESTUTILS_ASSERT_LIKE_FLAGS("comment block between fields", "#XferCommand.*\n.*#CleanMethod = KeepInstalled\n"s, std::regex::extended,
+        options->fields[1].precedingCommentBlock);
+    CPPUNIT_ASSERT_EQUAL("# inline comment"s, options->fields[1].followingInlineComment);
+    CPPUNIT_ASSERT_EQUAL("CheckSpace"s, options->fields[3].key);
+    CPPUNIT_ASSERT_EQUAL(""s, options->fields[3].value);
+    CPPUNIT_ASSERT_MESSAGE("no value present", !(options->fields[3].flags & IniFileFieldFlags::HasValue));
+    TESTUTILS_ASSERT_LIKE_FLAGS("empty lines in comments preserved", "\n# Pacman.*\n.*\n\n#NoUpgrade   =\n.*#TotalDownload\n"s, std::regex::extended,
+        options->fields[3].precedingCommentBlock);
+    CPPUNIT_ASSERT_EQUAL(""s, options->fields[3].followingInlineComment);
+    auto extraScope = ini.findSection(options, "extra");
+    CPPUNIT_ASSERT(extraScope != ini.sectionEnd());
+    CPPUNIT_ASSERT_EQUAL_MESSAGE("comment block which is only an empty line", "\n"s, extraScope->precedingCommentBlock);
+    CPPUNIT_ASSERT_EQUAL_MESSAGE("inline comment after scope", "# an inline comment after a scope name"s, extraScope->followingInlineComment);
+    CPPUNIT_ASSERT_EQUAL(1_st, extraScope->fields.size());
+    CPPUNIT_ASSERT(ini.sections.back().flags & IniFileSectionFlags::Implicit);
+    TESTUTILS_ASSERT_LIKE_FLAGS("comment block after last field present in implicitly added last scope", "\n# If you.*\n.*custompkgs\n"s,
+        std::regex::extended, ini.sections.back().precedingCommentBlock);
+
+    // test finding a field from file level and const access
+    const auto *const constIniFile = &ini;
+    auto includeField = constIniFile->findField("extra", "Include");
+    CPPUNIT_ASSERT(includeField.has_value());
+    CPPUNIT_ASSERT_EQUAL("Include"s, includeField.value()->key);
+    CPPUNIT_ASSERT_EQUAL("/etc/pacman.d/mirrorlist"s, includeField.value()->value);
+    CPPUNIT_ASSERT_MESSAGE("field not present", !constIniFile->findField("extra", "Includ").has_value());
+    CPPUNIT_ASSERT_MESSAGE("scope not present", !constIniFile->findField("extr", "Includ").has_value());
+
+    // write values again; there shouldn't be a difference as the parser and the writer are supposed to
+    // preserve the order of all elements and comments
+    std::stringstream newFile;
+    ini.make(newFile);
+    std::string originalContents;
+    inputFile.clear();
+    inputFile.seekg(std::ios_base::beg);
+    originalContents.assign((istreambuf_iterator<char>(inputFile)), istreambuf_iterator<char>());
+    CPPUNIT_ASSERT_EQUAL(originalContents, newFile.str());
+}
+
 /*!
  * \brief Tests CopyHelper.
  */
@@ -365,6 +439,19 @@ void IoTests::testReadFile()
 #endif
 }
 
+/*!
+ * \brief Tests writeFile().
+ */
+void IoTests::testWriteFile()
+{
+    const string path(workingCopyPath("test.ini", WorkingCopyMode::NoCopy));
+    writeFile(path, "some contents");
+    CPPUNIT_ASSERT_EQUAL("some contents"s, readFile(path));
+}
+
+/*!
+ * \brief Tests formatting functions of CppUtilities::EscapeCodes namespace.
+ */
 void IoTests::testAnsiEscapeCodes()
 {
     stringstream ss1;
@@ -427,12 +514,12 @@ void IoTests::testNativeFileStream()
     } catch (const std::ios_base::failure &failure) {
 #ifdef PLATFORM_WINDOWS
 #ifdef CPP_UTILITIES_USE_BOOST_IOSTREAMS
-        CPPUNIT_ASSERT_EQUAL("CreateFileW failed: iostream error"s, string(failure.what()));
+        TESTUTILS_ASSERT_LIKE("expected error with some message", "CreateFileW failed: .+", failure.what());
 #else
-        CPPUNIT_ASSERT_EQUAL("_wopen failed: iostream error"s, string(failure.what()));
+        TESTUTILS_ASSERT_LIKE("expected error with some message", "_wopen failed: .+", failure.what());
 #endif
 #else
-        CPPUNIT_ASSERT_EQUAL("open failed: iostream error"s, string(failure.what()));
+        TESTUTILS_ASSERT_LIKE("expected error with some message", "open failed: .+", failure.what());
 #endif
     }
     fileStream.clear();
